# -*- coding: utf-8 -*-
import re
from pathlib import Path

import httpx
from parsel import Selector
from yarl import URL

# TODO: Lots of repeated code. Create a class so parameters like yt_url is defined once and used and accessed by methods within the class.
# TODO: Think up another name for the class and functions as well.
# TODO: URL to define and delegate each part for each method.
# TODO: URL view the docs due to their may be a cache: https://tinyurl.com/2ctm95fo


def exception_duplicate_url(exception, message):
    """
    This is a quick base exception to raise duplicate url in newsboat.
    """
    return exception(message)


class Getting:
    """
    Basing fucntions to collect YouTube urls.
    """

    def __init__(self, yt_url):
        self.yt_url: URL = URL(yt_url)

    def getting_link(self):
        """
        This function will pull out the YouTube RSS link.
        """
        # TODO: Create a check that a valid YouTube url has been passed
        # TODO: if 'videos' not in

        if self.yt_url.name != "videos":
            # TODO: Remove this error and just add /videos to the YouTube url passed in.
            # TODO: You can join path of videos like: self.yt_url / 'videos'
            raise ValueError(
                """
                    YouTube url must have videos at the end of the url to get the RSS
                    feed.
                    Make sure to go to the creators page and click on their videos tab.
                    Copy/Paste that into getting_link to avoid this error.
                    """,
            )

        response = httpx.get(self.yt_url.human_repr())

        txt = response.text

        select = Selector(text=txt)

        return select.xpath('//link[@title="RSS"]/@href').get()

    def getting_name(self):
        """
        Getting channel name.
        """

        return self.yt_url.path.split("/")[1]

    def channel_name(self):
        """
        This function returns the name of the YouTube name to attach.
        """

        return self.yt_url.parts[1].replace("@", "")


class Duplicate:
    """
    Quick check that video url not a duplicate.
    """

    # TODO: Create test following typer test document!
    NEWSBOAT_URLS = Path("/Users/evanbaird/.newsboat/urls")

    def __init__(self, the_check):
        self.the_check = the_check

    def check(self):
        url = Getting(self.the_check)

        if re.findall(
            f"~.*{url.channel_name()}",
            Duplicate.NEWSBOAT_URLS.read_text(),
        ):
            raise exception_duplicate_url(
                ValueError,
                f"{url.channel_name()} is already in your urls list.",
            )



# -*- coding: utf-8 -*-
from pathlib import Path
from typing import List, Optional

import typer
from rich import print
from typing_extensions import Annotated

from .__init__ import __version__

# from .getting import Duplicate, getting_link, getting_name, channel_name
from .getting import Duplicate, Getting

app = typer.Typer(rich_markup_mode="rich")


def version_callback(value: bool):
    if value:
        print(
            f":sparkle:[bold]Awesome CLI Version:[/] [green]{__version__}[/]",
        )
        raise typer.Exit()


@app.command(epilog="[bold red]Made with desperation[/] :sweat:")
def main(
    urls: List[str],
    version: Annotated[
        Optional[bool],
        typer.Option(
            "--version",
            "-v",
            callback=version_callback,
            is_eager=True,
        ),
    ] = None,
):
    """
    Pass in a YouTube videos url and it will add the correct RSS feed to
    newsboat.
    """

    for url in urls:

        # TODO: Remember to follow typer raise doc: https://bit.ly/4hVgdme
        dupe_check = Duplicate(url)

        if dupe_check.check():
            print("This channel is already in your list!")
            raise typer.Exit(code=1)

        u = Getting(url)


        link = u.getting_link()

        newsboat = Path("/Users/evanbaird/.newsboat/urls")

        with newsboat.open(mode="a", encoding="utf-8") as wr:
            wr.write(f'\n{link} "~{u.channel_name()}"')

        print(
            f"{u.getting_name()} channel has been added to newsboat urls for you.",
        )



# -*- coding: utf-8 -*-
"""
Exceptions List: Follow this guide https://bit.ly/493plRK to creating a custom exception
Remember to test it!!!
"""


# TODO: We'll make this into a class and update it to just pass the print message only.
def exception_duplicate_url(exception, message):
    """
    This is a quick base exception to raise duplicate url in newsboat.
    """
    return exception(message)
